import pyutilib.workflow

# @ex:
class TaskF1(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('a',)
        self.inputs.declare('aval')
        self.outputs.declare('a', self.inputs.a)
        self.outputs.declare('aval', self.inputs.aval)

    def execute(self):
        pass

class TaskF2(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('A',)
        self.inputs.declare('Aval')
        self.outputs.declare('A', self.inputs.A)
        self.outputs.declare('Aval', self.inputs.Aval)

    def execute(self):
        pass

class TaskG(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x', action='map')
        self.inputs.declare('y', action='map')
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = {}
        for key in self.x:
            self.z[ self.x[key] ] = self.y[key]

F1 = TaskF1()
F2 = TaskF2()
G = TaskG()
G.inputs.x = F1.outputs.a
G.inputs.y = F1.outputs.aval
G.inputs.x = F2.outputs.A
G.inputs.y = F2.outputs.Aval

w = pyutilib.workflow.Workflow()
w.add(G)
print("IGNORE %s" % str(w(a='a', aval=1, A='A', Aval=2)))
# @:ex



import pyutilib.workflow
import os.path
import os
currdir = os.path.dirname(os.path.abspath(__file__))+os.sep

import sys
if sys.platform.startswith('win'):
    INPUT = open('example7a.txt','r')
    for line in INPUT:
        sys.stdout.write(line)
    INPUT.close()
else:

# @ex:
  class TaskH(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('dir')
        self.outputs.declare('list')
        self.add_resource(pyutilib.workflow.ExecutableResource(executable='/bin/ls'))

    def execute(self):
        self.resource('ls').run(self.dir, logfile=currdir+'logfile', debug=True)
        self.list = []
        INPUT = open(currdir+'logfile','r')
        for line in INPUT:
            self.list.append( line.strip() )
        INPUT.close()
        self.list.sort()

  H = TaskH()
  w = pyutilib.workflow.Workflow()
  w.add(H)
  print(w(dir=currdir+'dummy'))
# @:ex

  if os.path.exists(currdir+'logfile'):
    os.remove(currdir+'logfile')



import pyutilib.workflow

# @code:
class TaskA(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x', constant=True)
        self.inputs.declare('y')
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = self.x + self.y

class TaskZ(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('z', action='store_any')
        self.outputs.declare('z', self.inputs.z)

    def execute(self):
        pass

B = pyutilib.workflow.TaskFactory('workflow.switch')
A1 = TaskA()
A1.inputs.x = 1
B.add_branch('a', A1)
A2 = TaskA()
A2.inputs.x = -2
B.add_branch('b', A2)
Z = TaskZ()
Z.inputs.z = A1.outputs.z
Z.inputs.z = A2.outputs.z
w = pyutilib.workflow.Workflow()
w.add(B)

print("Branch a")
print(w(value='a', y=100))
w.reset()
print("Branch b")
print(w(value='b', y=100))
# @:code

try:
    print(w(value='c', y=100))
except ValueError:
    pass




import pyutilib.workflow

# @ex:
class TaskAA(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x')
        self.inputs.declare('y')
        self.add_argument('--x', dest='x', type=int)
        self.add_argument('--y', dest='y', type=int)
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = self.x + self.y

AA = TaskAA()
w = pyutilib.workflow.Workflow()
w.add(AA)
w.set_options(['--x=1', '--y=3', '--bad=4'])
print(w())
# @:ex



import pyutilib.workflow
import pyutilib.component.core

# @class:
class PluginTaskZ(pyutilib.workflow.TaskPlugin):

    pyutilib.component.core.alias('TaskZ')

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x')
        self.inputs.declare('y')
        self.add_argument('--x', dest='x', type=int)
        self.add_argument('--y', dest='y', type=int)
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = self.x + self.y

class PluginTaskY(pyutilib.workflow.TaskPlugin):

    pyutilib.component.core.alias('TaskY')

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('X')
        self.inputs.declare('Y')
        self.add_argument('--X', dest='X', type=int)
        self.add_argument('--Y', dest='Y', type=int)
        self.outputs.declare('Z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.Z = self.X * self.Y
# @:class



import pyutilib.workflow

class TaskA(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x')
        self.inputs.declare('y')
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = self.x + self.y

class TaskC(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('X')
        self.inputs.declare('y')
        self.outputs.declare('W')
        self.outputs.declare('Z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.W = self.X+self.y
        self.Z = 2*self.W

# @ex:
A = TaskA()
C = TaskC()
A.inputs.x = C.outputs.Z

w1 = pyutilib.workflow.Workflow()
w1.add(A)

AA = TaskA()
AA.inputs.x = w1.outputs.W
AA.inputs.y = w1.outputs.z

w2 = pyutilib.workflow.Workflow()
w2.add(AA)

print(w2(X=1, y=3))
# @:ex



import pyutilib.workflow

class TaskA(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x')
        self.inputs.declare('y')
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = self.x + self.y

# @ex:
A = TaskA()
w = pyutilib.workflow.Workflow()
w.add(A)
A.inputs.x = 1
A.inputs.y = 3
print(w())
# @:ex



from pyutilib.workflow import Resource, Task, Workflow

class BusyResource(Resource):

    def __init__(self, name=None):
        Resource.__init__(self)
        self._counter = 1

    def available(self):
        if self._counter > 0:
            print("BUSY %d" % self._counter)
            self._counter -= 1
            return False
        return True

class TaskA(Task):

    def __init__(self, *args, **kwds):
        Task.__init__(self, *args, **kwds)
        self.inputs.declare('x')
        self.outputs.declare('x', self.inputs.x)

    def execute(self):
        pass

A = TaskA()
A.add_resource(BusyResource())
w = Workflow()
w.add(A)

print(w(x=1))



import os

def strip_example(file):
    base, name = os.path.split(file)
    prefix = os.path.splitext(name)[0]
    if prefix.endswith('_strip'):
        return
    print "STRIPPING",file
    OUTPUT = open(base+'/'+prefix+'_strip.py','w')
    INPUT = open(file,'r')
    for line in INPUT:
        if line[0] == '#' and '@' in line:
            continue
        print >>OUTPUT, line,
    INPUT.close()
    OUTPUT.close()


for root, dirs, files in os.walk(os.path.abspath(os.path.dirname(__file__)), topdown=True):
    for name in files:
        if name == 'example_exec.py':
            continue
        if name.endswith('.py'):
            strip_example(root+'/'+name)




import sys
from math import sqrt
import pyutilib.pyro

if sys.version_info >= (3,0):
    xrange = range


def factorize(n):
    def isPrime(n):
        return not [x for x in xrange(2,int(sqrt(n))+1) if n%x == 0]
    primes = []
    candidates = xrange(2,n+1)
    candidate = 2
    while not primes and candidate in candidates:
        if n%candidate == 0 and isPrime(candidate):
            primes = primes + [candidate] + factorize(n/candidate)
        candidate += 1
    return primes


class FactorWorker(pyutilib.pyro.TaskWorker):

    def process(self,data):
        print "factorizing",data,"-->",
        sys.stdout.flush()
        res = factorize(int(data))
        print res
        return res



import pyutilib.workflow
import tasks_yz

driver = pyutilib.workflow.TaskDriver(prog='myprog',
   description='This is the description of this task driver',
   epilog="""**********************
This is more text
that describes this command driver.  Note

that the format of the epilog string is preserved in the
help
output!
**********************
""")
driver.register_task('TaskZ')
driver.register_task('TaskY')

print(driver.parse_args())



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________
#

import pyutilib.autotest
from pyutilib.component.core import alias
import pyutilib.subprocess


class ExampleTestDriver(pyutilib.autotest.TestDriverBase):
    """
    This test driver executes a unix command and compares its output
    with a baseline value.
    """

    alias('example')

    def run_test(self, testcase, name, options):
        """Execute a single test in the suite"""
        name = options.suite+'_'+name
        cmd = options.solver+' '
        if not options.cat_options is None:
            cmd += options.cat_options+' '
        cmd += options.file
        print( "Running test suite '%s'  test '%s'  command '%s'" % \
                (options.suite, name, cmd))
        pyutilib.subprocess.run(cmd, outfile=options.currdir+'test_'+name+".out")
        testcase.failUnlessFileEqualsBaseline(
                options.currdir+'test_'+name+".out",
                options.currdir+'test_'+name+".txt")



# Imports
import pyutilib.th as unittest
import glob
import os
from os.path import dirname, abspath, abspath, basename
import sys

currdir = dirname(abspath(__file__))+os.sep
datadir = currdir

def filter(line):
    return 'Running' in line or "IGNORE" in line or line.startswith('usage:') or 'Sub-commands' in line

# Declare an empty TestCase class
class Test(unittest.TestCase): pass

# Find all example*.py files, and use them to define baseline tests
for file in glob.glob(datadir+'example*.py'):
    bname = basename(file)
    name=bname.split('.')[0]
    #
    # We use add_baseline_test instead of add_import_test because the latter does not seem to
    # work when running with nosetests
    #
    if not os.path.exists(datadir+name+'.txt'):
        sys.stderr.write("WARNING:  no baseline available for file "+file)
    else:
        Test.add_import_test(name=name, cwd=datadir, baseline=datadir+name+'.txt', filter=filter)

if not sys.platform.startswith('win'):
    # Find all *.sh files, and use them to define baseline tests
    for file in glob.glob(datadir+'*.sh'):
        bname = basename(file)
        name=bname.split('.')[0]
        Test.add_baseline_test(cmd='cd %s; /usr/bin/env sh %s' % (datadir, file),  baseline=datadir+name+'.txt', name=name, filter=filter)

# Execute the tests
if __name__ == '__main__':
    unittest.main()



import pyutilib.workflow
import pyutilib.component.core

# @class:
class PluginTaskA(pyutilib.workflow.TaskPlugin):

    pyutilib.component.core.alias('TaskA')

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x')
        self.inputs.declare('y')
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = self.x + self.y
# @:class

# @usage:
A = pyutilib.workflow.TaskFactory('TaskA')
w = pyutilib.workflow.Workflow()
w.add(A)
print(w(x=1, y=3))
# @:usage



from pyutilib.component.core import *


class IPackage3Util(Interface):
    """Interface for Package3 utilities"""


class Package3Util(Plugin):

    implements(IPackage3Util)


inst1 = Package3Util()
inst2 = Package3Util()



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

from setuptools import setup

setup(
    name='Package1',
    version='0.1',
    packages=['package1'],
    package_data={'package1': []},
    author='Jane Doe',
    author_email='jdoe@dev.null',
    description='Package1 description.',
    license='BSD',
    keywords='package1 plugin',
    classifiers=[
        'Framework :: Package1',
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],

    #install_requires = ['Trac>=0.11', 'Genshi>=0.5'],
    entry_points={
        'project1.plugins': [
            'package1.main = package1.main',
        ]
    })



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

"""
Setup for PyUtilib package
"""

import os
from setuptools import setup, find_packages

def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as README:
        # Strip all leading badges up to, but not including the COIN-OR
        # badge so that they do not appear in the PyPI description
        while True:
            line = README.readline()
            if 'COIN-OR' in line:
                break
            if line.strip() and '[![' not in line:
                break
        return line + README.read()

requires=[ 'nose', 'six' ]

setup(name="PyUtilib",
    version='6.0.0',
    maintainer='William E. Hart',
    maintainer_email='wehart@sandia.gov',
    url = 'https://github.com/PyUtilib/pyutilib',
    license = 'BSD',
    platforms = ["any"],
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    description = 'PyUtilib: A collection of Python utilities',
    long_description = read('README.md'),
    long_description_content_type='text/markdown',
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: Jython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'],
      packages=find_packages(),
      keywords=['utility'],
      install_requires=requires,
      entry_points="""
        [nose.plugins.0.10]
        nose.testdata = pyutilib.th.nose_testdata:TestData
        nose.forcedgc = pyutilib.th.nose_gc:ForcedGC
        nose.timeout = pyutilib.th.nose_timeout:TestTimeout
        [console_scripts]
        test.pyutilib = pyutilib.dev.runtests:runPyUtilibTests
        lbin = pyutilib.dev.lbin:main
        lpython = pyutilib.dev.lpython:main
        pypi_downloads = pyutilib.dev.pypi_downloads:main
        replaceCopyright = pyutilib.dev.replaceCopyright:main
        checkCopyright = pyutilib.dev.checkCopyright:main
        pyutilib_test_driver = pyutilib.autotest.driver:main
        dispatch_srvr=pyutilib.pyro.dispatch_srvr:main
      """
      )




import pyutilib.workflow

# @code:
class TaskA(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x')
        self.inputs.declare('y')
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = self.x + self.y

B = pyutilib.workflow.TaskFactory('workflow.selection')
A = TaskA()
A.inputs.x = B.outputs.selection
w = pyutilib.workflow.Workflow()
w.add(B)

print(w(index='a', y=100, data={'a':1, 'b':2}))
w.reset()
print(w(index='b', y=100, data={'a':1, 'b':2}))
# @:code



import pyutilib.workflow

# @class:
class TaskA(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('z')
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = -1*self.z


class TaskB(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('y')
        self.outputs.declare('y')

    def execute(self):
        """Compute the sum of the inputs."""
        self.y = -1*self.y

class TaskC(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x', action='store_any')
        self.outputs.declare('x')

    def execute(self):
        pass
# @:class

# @usage:
A = TaskA()
B = TaskB()

C = pyutilib.workflow.TaskFactory('workflow.branch')
C.add_branch(True, A)
C.add_branch(False, B)

D = TaskC()
D.inputs.x = A.outputs.z
D.inputs.x = B.outputs.y

w = pyutilib.workflow.Workflow()
w.add(C)

print(w(value=True, z=1, y=2))
w.reset()
print(w(value=False, z=1, y=2))
# @:usage



import pyutilib.workflow

class TaskA(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x')
        self.inputs.declare('y')
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = self.x + self.y

# @class:
class TaskD(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('X')
        self.inputs.declare('y')
        self.inputs.declare('a', constant=True)
        self.outputs.declare('W')
        self.outputs.declare('Z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.W = self.X+self.y+self.a
        self.Z = 2*self.W
# @:class

A = TaskA()
D = TaskD()
D.inputs.a = 100
A.inputs.x = D.outputs.Z

w = pyutilib.workflow.Workflow()
w.add(A)
print(w(X=1, y=3))



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________
#
#
# Tests driven by example1.yml
#

import os
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep

import pyutilib.th as unittest
import pyutilib.autotest

pyutilib.autotest.create_test_suites(filename=currdir+'example1.yml', _globals=globals())

if __name__ == "__main__":
    unittest.main()



import pyutilib.workflow
import math

# @class:
class TaskD(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x', action='append')
        self.outputs.declare('z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.z = sum(self.x)
# @:class

# @usage:
class TaskE(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        """Constructor."""
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('Y')
        self.outputs.declare('Z')

    def execute(self):
        """Compute the sum of the inputs."""
        self.Z = int(math.floor(self.Y/7.0))

D = TaskD()

# there is currently an issue with memory leaks and the workflow/task/port structure.
# to correct this issue in the interim, all tasks that are to be retained in memory
# (not garbage collected) should be referenced by the user. thus, the introduction of 
# e_tasks. ultimately, the workflow should own a reference to all composite tasks.
e_tasks = []

for i in range(100):
    E = TaskE()
    e_tasks.append(E)
    D.inputs.x = E.outputs.Z

w = pyutilib.workflow.Workflow()
w.add(D)
print(w(Y=100))
# @:usage



import pyutilib.workflow
import pyutilib.component.core

# @usage:
import tasks_yz

driver = pyutilib.workflow.TaskDriver()
driver.register_task('TaskZ')
driver.register_task('TaskY')

print(driver.parse_args(['TaskZ','--x=3','--y=4']))
print(driver.parse_args(['TaskY','--X=3','--Y=4']))
# @:usage
